#include <vector>
#include <string.h>
#include <iostream>

#include "rapidxml-1.13/rapidxml.hpp"
#include "rapidxml-1.13/rapidxml_utils.hpp"

#include "xml2latex.hpp"

#ifndef namespace_rapidxml
#define namespace_rapidxml
using namespace rapidxml;
#endif

#ifndef namespace_std
#define namespace_std
using namespace std;
#endif

MyRapid::MyRapid(string file,string extension)
{
    XML = new xml_document<>();
    if (!read_xml_file(file,extension))
    {
        cout << "Unable to open " <<  file+ extension;
        exit(1);
    }
}
MyRapid::~MyRapid()
{
    XML->clear();
    delete XML;
    delete XMLvect;
}


bool MyRapid::read_xml_file(string file, string extension)
{
    string file_ext = file + extension;
    char *cstr = new char[file_ext.length() + 1];
    strcpy(cstr, file_ext.c_str());
    ifstream XmlF (cstr);
    if (!XmlF.good())
    {
        cout << "file "<< file_ext<< " no exists";
        return false;
    }
    XMLvect =new vector<char>((istreambuf_iterator<char>(XmlF)), istreambuf_iterator<char>());
    XMLvect->push_back('\0');
    try
    {
        XML->parse<0>(&(*XMLvect)[0]);
    }
    catch( parse_error p )
    {
        cout <<"|" <<p.what()<<"|";
        cout << p.where<char>();
        return false;
    }
    delete cstr;
    return true;
};

Xnode::Xnode(xml_node<> *org):Org(org) {};

Xnode::~Xnode()
{
    for (int i=0; i<XList.size(); i++)
    {
        delete XList[i];
    }
}
void Xnode::Xml2LaTeX(ostream & file,int kind)
{
    if (kind==0)
    {
        if (Org)
        {
            file << "\\"<<Org->name() << "{"<<Org->value()<<"}";
        }
        else
        {
            file <<"???"<<endl;
        }
    }
    else
    {
        for(int i=0; i<XList.size(); i++)
        {
            if (kind ==1)
            {
                file << " & ";
            }
            else if (kind ==2)file << "{";
            XList[i]->Xml2LaTeX(file,0);
            if (kind ==2) file << "}";
        }
    }

}

TYPE::TYPE(string name):Xnode(NULL)
{
    Name=name;
}

void TYPE::Xml2LaTeX(ostream & file,int kind)
{
    switch (kind)
    {
    case 0:
        file << "\\begin{tabular}{c|rrrrrr}"<<endl;
        file << "Nr & Common & Botanic & Zone & Light & Price & Availabity\\\\\\hline"<<endl;
        for(int i=0; i< XList.size(); i++)
        {
            file << i+1;
            XList[i]->Xml2LaTeX(file,1);
            file<<"\\\\"<<endl;
        }
        file << "\\end{tabular}"<<endl;
        break;
    case 1:
        file<<"\\begin{enumerate}"<<endl;
        for(int i=0; i< XList.size(); i++)
        {
            file << "\\item\\Food";
            XList[i]->Xml2LaTeX(file,2);
            file<<endl;
        }
        file <<"\\end{enumerate}"<<endl;
        break;

    case 2:
        file <<"\\begin{itemize}"<<endl;
        for(int i=0; i< XList.size(); i++)
        {
            file << "\\item[]\\Book";
            XList[i]->Xml2LaTeX(file,2);
            file<<endl;
        }
        file <<"\\end{itemize}"<<endl;
    }
};

_DATE::_DATE(xml_node<> *org):Xnode(org)
{
    char * V = org->value();
    int i=0, L= strlen(V);
    D=M=Y="";
    while (i<L && (V[i]!='-'))
    {
        Y += V[i++];
    }
    if (i++==L) cout << "date error 1";
    while (i<L && (V[i]!='-'))
    {
        D += V[i++];
    }
    if (i++==L) cout << "date error 2";
    while (i<L && (V[i]!='-'))
    {
        M += V[i++];
    }
}
void _DATE::Xml2LaTeX(ostream & file,int kind)
{
    file<<"\\DATE{"<<D<<"}{"<<M<<"}{"<<Y<<"}";
}

_CATALOG::_CATALOG(xml_node<> *org): Xnode(org)
{
    DATA[0]=DATA[1]=DATA[2]=NULL;
    xml_node<> * Ch = org->first_node();
    while (Ch)
    {

        if (strcmp(Ch->name(),(char*)"PLANT")==0)
        {
            if (DATA[0]==NULL) DATA[0]=new TYPE("Plants -- Table");
            DATA[0]->XList.push_back(new _PLANT(Ch));
        }
        else if (strcmp(Ch->name(),(char*)"FOOD")==0)
        {
            if (DATA[1]==NULL) DATA[1]=new TYPE("My List of Foods");
            DATA[1]->XList.push_back(new _FOOD(Ch));
        }
        else if (strcmp(Ch->name(),(char*)"BOOK")==0)
        {
            if (DATA[2]==NULL) DATA[2]=new TYPE("My Books");
            DATA[2]->XList.push_back(new _BOOK(Ch));
        }
        else
            cout << "unknown type: "<<Ch->name()<<endl;
        Ch=Ch->next_sibling();
    }
}

void _CATALOG::Xml2LaTeX(ostream & file,int kind)
{
    for(int i=0; i < 3; i++)
    {
        if (DATA[i]!=NULL)
        {
            file << "\\section{" << DATA[i]->Name<<"}"<<endl;
            DATA[i]->Xml2LaTeX(file,i);
        }
    }
}

_PLANT::_PLANT(xml_node<> *org):Xnode(org)
{
    xml_node<>* N=Org->first_node((char*)"COMMON");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 1";

    //mozna wykorzysta N->next_sibling(),
    //ale wtedy musimy by pewni kolejnosci wzw w .xml

    N=Org->first_node((char*)"BOTANICAL");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 2";
    N=Org->first_node((char*)"ZONE");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 3";
    N=Org->first_node((char*)"LIGHT");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 4";
    N=Org->first_node((char*)"PRICE");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 5";
    N=Org->first_node((char*)"AVAILABILITY");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 6";
};

_FOOD::_FOOD(xml_node<> *org):Xnode(org)
{
    xml_node<>* N=Org->first_node((char*)"NAME");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 7";
    N=Org->first_node((char*)"PRICE");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 8";
    N=Org->first_node((char*)"DESCRIPTION");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 9";
    N=Org->first_node((char*)"CALORIES");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 10";
};

_BOOK::_BOOK(xml_node<> *org):Xnode(org)
{
    xml_node<>* N=Org->first_node((char*)"AUTHOR");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 11";
    N=Org->first_node((char*)"TITLE");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 12";
    N=Org->first_node((char*)"GENER");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 13";
    N=Org->first_node((char*)"PRICE");
    if (N)
    {
        XList.push_back(new Xnode(N));
    }
    else cout << "Parser Error 14";
    N=Org->first_node((char*)"DATE");
    if (N)
    {
        XList.push_back(new _DATE(N));
    }
    else cout << "Parser Error 14";
};

Xml2Latex::Xml2Latex(string FileXML)
{
    DataXML = new MyRapid(FileXML,".xml");

    if (DataXML->XML && DataXML->XML->first_node() &&
            strcmp(DataXML->XML->first_node()->name(),(char*)"CATALOG")==0)
    {
        Catalog = new _CATALOG(DataXML->XML->first_node());
    }
    else exit(-20);
}
Xml2Latex::~Xml2Latex()
{

    delete DataXML;
    delete Catalog;
}


void Xml2Latex::ADDpreamble(ostream & file)
{
    file <<"\\documentclass{article}"<<endl;
    file <<"\\usepackage{eurosym} \\usepackage{amsmath,amsthm,amssymb}"<<endl;
    file<< "\\usepackage{polski} \\usepackage[utf8]{inputenc}"<<endl;
    file <<"\\newcommand{\\COMMON}[1]{#1}"<<endl;
    file <<"\\newcommand{\\BOTANICAL}[1]{{\\textit{#1}}}"<<endl;
    file <<"\\newcommand{\\ZONE}[1]{\\textbf{\\expandafter{\\romannumeral #1\\relax}}}"<<endl;
    file <<"\\newcommand{\\LIGHT}[1]{{\\emph{#1}}}"<<endl;
    file <<"\\newcommand{\\PRICE}[1]{{#1} \\euro}"<<endl;
    file <<"\\newcommand{\\AVAILABILITY}[1]{{#1}}"<<endl;
    file <<"\\newcommand{\\NAME}[1]{\\textbf{#1}}"<<endl;
    file <<"\\newcommand{\\DESCRIPTION}[1]{\\textit{#1}}"<<endl;
    file <<"\\newcommand{\\CALORIES}[1]{{#1}kcal}"<<endl;
    file <<"\\newcommand{\\AUTHOR}[1]{{#1}}"<<endl;
    file <<"\\newcommand{\\TITLE}[1]{{#1}}"<<endl;
    file <<"\\newcommand{\\GENER}[1]{$\\mathfrak{#1}$}"<<endl;
    file <<"\\newcommand{\\Food}[4]{{#1}, {#4}, Price: {#2}.\\begin{center}{#3}\\end{center}}"<<endl;
    file <<"\\newcommand{\\Book}[5]{$\\mkern-25mu${#1}, {#2}. Thematic category: {#3}. Purchased on: {#5}. Price: {#4}.}"<<endl;
    file <<"\\newcommand{\\DATE}[3]{{#1}.{{\\uppercase\\expandafter{\\romannumeral #2\\relax}}}.{#3}}"<<endl;
}

void Xml2Latex::CreateText()
{
    ofstream Tex("wynik.tex");
    ADDpreamble(Tex);
    Tex << "\\begin{document}"<<endl;
    Catalog->Xml2LaTeX(Tex,0);
    Tex << "\\end{document}"<<endl;
    Tex.flush();
    Tex.close();
}



